/*
	Copyright(c) 2011 Vincent Christlein <vincent.christlein@cs.fau.de>.

	This file may be licensed under the terms of of the GNU General Public
	License, version 3, as published by the Free Software Foundation. You can
	find it here: http://www.gnu.org/licenses/gpl.html
*/

#ifndef MARK_H
#define MARK_H

#include "cv.h"
#include "copymoveframework_config.h"
#include <map>
#include <vector>
#include <string>

// Forward declarations
class Matching;
/*! \brief marks somehow the copied block pairs in the image
 */
class Mark
{
public:
	Mark(cv::Mat & _img,
		 cv::Mat & _origimg,
		 struct mark_cfg &_config,
		 const std::vector<cv::Mat_<cv::Point> > & _corres,
		 int _blockSize);
	void markIt(cv::Mat& matrix);
	void markIt(std::vector< std::multimap<std::pair<int,int>,
				std::pair<cv::Point, cv::Point> > > shift);
	/// marks each region w. different color and shift-vecs if whished with green
	void markRegions(std::vector<std::vector<std::pair<cv::Point,cv::Point> > > & clusters,
					 bool draw_shift_vecs);
	void markContour(const cv::Mat& cmfd_map,
					 std::vector<std::vector<cv::Point> > * contours,
					 std::vector<cv::Vec4i> * hierarchy);
	/// marks the convex hull
	void markHull(const std::vector<std::vector<std::pair<cv::Point,cv::Point> > > & clusters);
	/// write pairs - marks pairs in GT differently
	void writePairs(const std::string & groundTruthFile, bool shift);
	/// write chromaticity matrix of positions
	void writeChrom(void);
	/// computes correlation map from transformations
	std::vector<cv::Mat_<uchar> > computeCorrelation(const std::vector<cv::Mat> & trans,
													 bool write);
	/// just writes out the matrix given, nothing fancy here
	static void writeMatrix(const cv::Mat & matrix);
	/// dumps a matrix with the unique identifier
	static void dumpMatrix(std::string identifier, const cv::Mat & dump_matrix);
private:
	cv::Mat_<float> computeCorrelMap(const cv::Mat_<uchar> &plane,
									 const cv::Mat &trans,
									 bool inverse = false);
	int block_size;
	cv::Mat & img;
	cv::Mat & origimg;
	struct mark_cfg config;
	const std::vector<cv::Mat_<cv::Point> > & corres;
	/// png-parameters
	std::vector<int> params;
};

#endif // MARK_H
